/*
 * RSSBot
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.rssbot

import com.kotlindiscord.kord.extensions.ExtensibleBot
import dev.kord.common.entity.PresenceStatus
import dev.kord.core.event.gateway.ReadyEvent
import dev.kord.core.on
import net.moonleay.rssbot.data.CredentialManager
import net.moonleay.rssbot.data.database.DB
import net.moonleay.rssbot.extensions.FeedExtension
import net.moonleay.rssbot.extensions.StuffExtension
import net.moonleay.rssbot.jobs.FeedUpdater
import net.moonleay.rssbot.jobs.component.JobManager
import net.moonleay.rssbot.util.Logger
import kotlin.system.exitProcess

object Bot {
    //The kord object gets set at app launch
    lateinit var bot: ExtensibleBot

    suspend fun start() {
        Logger.out("Starting Bot...")

        // Load config
        CredentialManager.load()

        // Don't run the bot when there is no bot token in config
        if (CredentialManager.token == "empty") {
            Logger.out("The config does not contain a bot token.")
            exitProcess(3)
        }

        // Check if the credentials for the Database are existent, don't run if they are missing
        if (CredentialManager.dbDomain == "empty" || CredentialManager.dbName == "empty" || CredentialManager.dbUser == "empty" || CredentialManager.dbPassword == "empty") {
            Logger.out("The config does not contain the whole Database credentials.")
            exitProcess(3)
        }

        // Connect to the database
        DB.connect(
            CredentialManager.dbDomain,
            CredentialManager.dbName,
            CredentialManager.dbUser,
            CredentialManager.dbPassword
        )
        // Make sure the database is up-to-date
        DB.register()

        // Register all the jobs
        JobManager.addJob(FeedUpdater)

        // Create the bot object
        bot = ExtensibleBot(CredentialManager.token) {
            applicationCommands {
                enabled = true
            }

            extensions {
                add(::FeedExtension)
                add(::StuffExtension)
            }

            this.presence {
                this.status = PresenceStatus.DoNotDisturb
                this.listening("rss feeds")
            }
        }

        bot.kordRef.on<ReadyEvent> {
            FeedUpdater.update(true)
        }


        //Start the bot
        bot.start()
    }

}
